import abc
from pong import directions_from_input
import copy


class Agent(abc.ABC):

    def __init__(self):
        pass

    @abc.abstractmethod
    def action(self, observation):
        raise NotImplementedError()

    def learn(self, reward):
        pass

    def epoch_end(self):
        pass

    def save(self, filename):
        pass

    def load(self, filename):
        pass

    def get_apply_agent(self):
        return copy.deepcopy(self)


class HumanAgent(Agent):

    def __init__(self):
        super().__init__()

    def action(self, observation):
        return directions_from_input(*observation)


class SimpleAgent(Agent):

    def __init__(self):
        super().__init__()

    def action(self, observation):
        # if paddle_frect.pos[1] + paddle_frect.size[1] / 2 < ball_frect.pos[1] + ball_frect.size[1] / 2:
        if observation[1] + 0.125 < observation[5] + 0.026785714:
            return "down"
        else:
            return "up"


class PolicyAgent(Agent):

    def __init__(self, policy):
        super().__init__()
        self.policy = policy

    def action(self, observation):
        return self.policy(observation)


